@@ -1,4 +1,4 @@
-# Copyright 2014 Google Inc.
+# Copyright 2014 Google LLC
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -15,36 +15,46 @@
 
 import os
 
-from google.cloud.proto.datastore.v1 import datastore_pb2 as _datastore_pb2
-
+import google.api_core.client_options
 from google.cloud._helpers import _LocalStack
-from google.cloud._helpers import (
-    _determine_default_project as _base_default_project)
+from google.cloud._helpers import _determine_default_project as _base_default_project
 from google.cloud.client import ClientWithProject
-from google.cloud.environment_vars import DISABLE_GRPC
-from google.cloud.environment_vars import GCD_DATASET
-from google.cloud.environment_vars import GCD_HOST
-
-from google.cloud.datastore._http import HTTPDatastoreAPI
+from google.cloud.datastore import __version__
 from google.cloud.datastore import helpers
+from google.cloud.datastore._http import HTTPDatastoreAPI
 from google.cloud.datastore.batch import Batch
 from google.cloud.datastore.entity import Entity
 from google.cloud.datastore.key import Key
 from google.cloud.datastore.query import Query
 from google.cloud.datastore.transaction import Transaction
+from google.cloud.environment_vars import DISABLE_GRPC
+from google.cloud.environment_vars import GCD_DATASET
+from google.cloud.environment_vars import GCD_HOST
+
 try:
-    from google.cloud.datastore._gax import make_datastore_api
-    _HAVE_GRPC = True
+    from google.cloud.datastore._gapic import make_datastore_api
+
 except ImportError:  # pragma: NO COVER
+    from google.api_core import client_info
+
     make_datastore_api = None
     _HAVE_GRPC = False
+    _CLIENT_INFO = client_info.ClientInfo(client_library_version=__version__)
+else:
+    from google.api_core.gapic_v1 import client_info
+
+    _HAVE_GRPC = True
+    _CLIENT_INFO = client_info.ClientInfo(
+        client_library_version=__version__, gapic_version=__version__
+    )
 
 
 _MAX_LOOPS = 128
 """Maximum number of iterations to wait for deferred keys."""
-_DATASTORE_BASE_URL = 'https://datastore.googleapis.com'
+_DATASTORE_BASE_URL = "https://datastore.googleapis.com"
 """Datastore API request URL base."""
 
+
 _USE_GRPC = _HAVE_GRPC and not os.getenv(DISABLE_GRPC, False)
 
 
@@ -79,16 +89,22 @@ def _determine_default_project(project=None):
     return project
 
 
-def _extended_lookup(datastore_api, project, key_pbs,
-                     missing=None, deferred=None,
-                     eventual=False, transaction_id=None):
+def _extended_lookup(
+    datastore_api,
+    project,
+    key_pbs,
+    missing=None,
+    deferred=None,
+    eventual=False,
+    transaction_id=None,
+):
     """Repeat lookup until all keys found (unless stop requested).
 
     Helper function for :meth:`Client.get_multi`.
 
     :type datastore_api:
         :class:`google.cloud.datastore._http.HTTPDatastoreAPI`
-        or :class:`google.cloud.datastore._gax.GAPICDatastoreAPI`
+        or :class:`google.cloud.datastore_v1.gapic.DatastoreClient`
     :param datastore_api: The datastore API object used to connect
                           to datastore.
 
@@ -123,19 +139,20 @@ def _extended_lookup(datastore_api, project, key_pbs,
              empty list.
     """
     if missing is not None and missing != []:
-        raise ValueError('missing must be None or an empty list')
+        raise ValueError("missing must be None or an empty list")
 
     if deferred is not None and deferred != []:
-        raise ValueError('deferred must be None or an empty list')
+        raise ValueError("deferred must be None or an empty list")
 
     results = []
 
     loop_num = 0
-    read_options = _get_read_options(eventual, transaction_id)
+    read_options = helpers.get_read_options(eventual, transaction_id)
     while loop_num < _MAX_LOOPS:  # loop against possible deferred.
         loop_num += 1
         lookup_response = datastore_api.lookup(
-            project, read_options, key_pbs)
+            project, key_pbs, read_options=read_options
+        )
 
         # Accumulate the new results.
         results.extend(result.entity for result in lookup_response.found)
@@ -166,10 +183,10 @@ class Client(ClientWithProject):
        >>> client = datastore.Client()
 
     :type project: str
-    :param project: (optional) The project to pass to proxied API methods.
+    :param project: (Optional) The project to pass to proxied API methods.
 
     :type namespace: str
-    :param namespace: (optional) namespace to pass to proxied API methods.
+    :param namespace: (Optional) namespace to pass to proxied API methods.
 
     :type credentials: :class:`~google.auth.credentials.Credentials`
     :param credentials: (Optional) The OAuth2 Credentials to use for this
@@ -177,6 +194,19 @@ class Client(ClientWithProject):
                         passed), falls back to the default inferred from the
                         environment.
 
+    :type client_info: :class:`google.api_core.gapic_v1.client_info.ClientInfo`
+                       or :class:`google.api_core.client_info.ClientInfo`
+    :param client_info: (Optional) The client info used to send a user-agent
+                        string along with API requests. If ``None``, then
+                        default info will be used. Generally,
+                        you only need to set this if you're developing your
+                        own library or partner tool.
+
+    :type client_options: :class:`~google.api_core.client_options.ClientOptions`
+                          or :class:`dict`
+    :param client_options: (Optional) Client options used to set user options on the
+                           client. API Endpoint should be set through client_options.
+
     :type _http: :class:`~requests.Session`
     :param _http: (Optional) HTTP object to make requests. Can be any object
                   that defines ``request()`` with the same interface as
@@ -195,14 +225,25 @@ class Client(ClientWithProject):
                       change in the future.
     """
 
-    SCOPE = ('https://www.googleapis.com/auth/datastore',)
+    SCOPE = ("https://www.googleapis.com/auth/datastore",)
     """The scopes required for authenticating as a Cloud Datastore consumer."""
 
-    def __init__(self, project=None, namespace=None,
-                 credentials=None, _http=None, _use_grpc=None):
+    def __init__(
+        self,
+        project=None,
+        namespace=None,
+        credentials=None,
+        client_info=_CLIENT_INFO,
+        client_options=None,
+        _http=None,
+        _use_grpc=None,
+    ):
         super(Client, self).__init__(
-            project=project, credentials=credentials, _http=_http)
+            project=project, credentials=credentials, _http=_http
+        )
         self.namespace = namespace
+        self._client_info = client_info
+        self._client_options = client_options
         self._batch_stack = _LocalStack()
         self._datastore_api_internal = None
         if _use_grpc is None:
@@ -211,15 +252,33 @@ class Client(ClientWithProject):
             self._use_grpc = _use_grpc
         try:
             host = os.environ[GCD_HOST]
-            self._base_url = 'http://' + host
+            self._base_url = "http://" + host
         except KeyError:
-            self._base_url = _DATASTORE_BASE_URL
+            api_endpoint = _DATASTORE_BASE_URL
+            if client_options:
+                if type(client_options) == dict:
+                    client_options = google.api_core.client_options.from_dict(
+                        client_options
+                    )
+                if client_options.api_endpoint:
+                    api_endpoint = client_options.api_endpoint
+            self._base_url = api_endpoint
 
     @staticmethod
     def _determine_default(project):
         """Helper:  override default project detection."""
         return _determine_default_project(project)
 
+    @property
+    def base_url(self):
+        """Getter for API base URL."""
+        return self._base_url
+
+    @base_url.setter
+    def base_url(self, value):
+        """Setter for API base URL."""
+        self._base_url = value
+
     @property
     def _datastore_api(self):
         """Getter for a wrapped API object."""
@@ -276,7 +335,7 @@ class Client(ClientWithProject):
         if isinstance(transaction, Transaction):
             return transaction
 
-    def get(self, key, missing=None, deferred=None, transaction=None):
+    def get(self, key, missing=None, deferred=None, transaction=None, eventual=False):
         """Retrieve an entity from a single key (if it exists).
 
         .. note::
@@ -302,15 +361,29 @@ class Client(ClientWithProject):
         :param transaction: (Optional) Transaction to use for read consistency.
                             If not passed, uses current transaction, if set.
 
+        :type eventual: bool
+        :param eventual: (Optional) Defaults to strongly consistent (False).
+                         Setting True will use eventual consistency, but cannot
+                         be used inside a transaction or will raise ValueError.
+
         :rtype: :class:`google.cloud.datastore.entity.Entity` or ``NoneType``
         :returns: The requested entity if it exists.
+
+        :raises: :class:`ValueError` if eventual is True and in a transaction.
         """
-        entities = self.get_multi(keys=[key], missing=missing,
-                                  deferred=deferred, transaction=transaction)
+        entities = self.get_multi(
+            keys=[key],
+            missing=missing,
+            deferred=deferred,
+            transaction=transaction,
+            eventual=eventual,
+        )
         if entities:
             return entities[0]
 
-    def get_multi(self, keys, missing=None, deferred=None, transaction=None):
+    def get_multi(
+        self, keys, missing=None, deferred=None, transaction=None, eventual=False
+    ):
         """Retrieve entities, along with their attributes.
 
         :type keys: list of :class:`google.cloud.datastore.key.Key`
@@ -331,10 +404,16 @@ class Client(ClientWithProject):
         :param transaction: (Optional) Transaction to use for read consistency.
                             If not passed, uses current transaction, if set.
 
+        :type eventual: bool
+        :param eventual: (Optional) Defaults to strongly consistent (False).
+                         Setting True will use eventual consistency, but cannot
+                         be used inside a transaction or will raise ValueError.
+
         :rtype: list of :class:`google.cloud.datastore.entity.Entity`
         :returns: The requested entities.
         :raises: :class:`ValueError` if one or more of ``keys`` has a project
                  which does not match our project.
+        :raises: :class:`ValueError` if eventual is True and in a transaction.
         """
         if not keys:
             return []
@@ -342,7 +421,7 @@ class Client(ClientWithProject):
         ids = set(key.project for key in keys)
         for current_id in ids:
             if current_id != self.project:
-                raise ValueError('Keys do not match project')
+                raise ValueError("Keys do not match project")
 
         if transaction is None:
             transaction = self.current_transaction
@@ -350,7 +429,8 @@ class Client(ClientWithProject):
         entity_pbs = _extended_lookup(
             datastore_api=self._datastore_api,
             project=self.project,
-            key_pbs=[k.to_protobuf() for k in keys],
+            key_pbs=[key.to_protobuf() for key in keys],
+            eventual=eventual,
             missing=missing,
             deferred=deferred,
             transaction_id=transaction and transaction.id,
@@ -358,16 +438,15 @@ class Client(ClientWithProject):
 
         if missing is not None:
             missing[:] = [
-                helpers.entity_from_protobuf(missed_pb)
-                for missed_pb in missing]
+                helpers.entity_from_protobuf(missed_pb) for missed_pb in missing
+            ]
 
         if deferred is not None:
             deferred[:] = [
-                helpers.key_from_protobuf(deferred_pb)
-                for deferred_pb in deferred]
+                helpers.key_from_protobuf(deferred_pb) for deferred_pb in deferred
+            ]
 
-        return [helpers.entity_from_protobuf(entity_pb)
-                for entity_pb in entity_pbs]
+        return [helpers.entity_from_protobuf(entity_pb) for entity_pb in entity_pbs]
 
     def put(self, entity):
         """Save an entity in the Cloud Datastore.
@@ -463,37 +542,43 @@ class Client(ClientWithProject):
                  partial key.
         """
         if not incomplete_key.is_partial:
-            raise ValueError(('Key is not partial.', incomplete_key))
+            raise ValueError(("Key is not partial.", incomplete_key))
 
         incomplete_key_pb = incomplete_key.to_protobuf()
         incomplete_key_pbs = [incomplete_key_pb] * num_ids
 
         response_pb = self._datastore_api.allocate_ids(
-            incomplete_key.project, incomplete_key_pbs)
-        allocated_ids = [allocated_key_pb.path[-1].id
-                         for allocated_key_pb in response_pb.keys]
-        return [incomplete_key.completed_key(allocated_id)
-                for allocated_id in allocated_ids]
+            incomplete_key.project, incomplete_key_pbs
+        )
+        allocated_ids = [
+            allocated_key_pb.path[-1].id for allocated_key_pb in response_pb.keys
+        ]
+        return [
+            incomplete_key.completed_key(allocated_id) for allocated_id in allocated_ids
+        ]
 
     def key(self, *path_args, **kwargs):
         """Proxy to :class:`google.cloud.datastore.key.Key`.
 
         Passes our ``project``.
         """
-        if 'project' in kwargs:
-            raise TypeError('Cannot pass project')
-        kwargs['project'] = self.project
-        if 'namespace' not in kwargs:
-            kwargs['namespace'] = self.namespace
+        if "project" in kwargs:
+            raise TypeError("Cannot pass project")
+        kwargs["project"] = self.project
+        if "namespace" not in kwargs:
+            kwargs["namespace"] = self.namespace
         return Key(*path_args, **kwargs)
 
     def batch(self):
         """Proxy to :class:`google.cloud.datastore.batch.Batch`."""
         return Batch(self)
 
-    def transaction(self):
-        """Proxy to :class:`google.cloud.datastore.transaction.Transaction`."""
-        return Transaction(self)
+    def transaction(self, **kwargs):
+        """Proxy to :class:`google.cloud.datastore.transaction.Transaction`.
+
+        :param kwargs: Keyword arguments to be passed in.
+        """
+        return Transaction(self, **kwargs)
 
     def query(self, **kwargs):
         """Proxy to :class:`google.cloud.datastore.query.Query`.
@@ -504,100 +589,103 @@ class Client(ClientWithProject):
 
         .. testsetup:: query
 
-           from google.cloud import datastore
+            import os
+            import uuid
 
-           client = datastore.Client()
-           query = client.query(kind='_Doctest')
+            from google.cloud import datastore
 
-           def do_something(entity):
-               pass
+            unique = os.getenv('CIRCLE_BUILD_NUM', str(uuid.uuid4())[0:8])
+            client = datastore.Client(namespace='ns{}'.format(unique))
+            query = client.query(kind='_Doctest')
+
+            def do_something(entity):
+                pass
 
         .. doctest:: query
 
-           >>> query = client.query(kind='MyKind')
-           >>> query.add_filter('property', '=', 'val')
+            >>> query = client.query(kind='MyKind')
+            >>> query.add_filter('property', '=', 'val')
 
         Using the query iterator
 
         .. doctest:: query
 
-           >>> query_iter = query.fetch()
-           >>> for entity in query_iter:
-           ...     do_something(entity)
+            >>> query_iter = query.fetch()
+            >>> for entity in query_iter:
+            ...     do_something(entity)
 
         or manually page through results
 
         .. testsetup:: query-page
 
-           from google.cloud import datastore
-           from tests.system.test_system import Config  # system tests
+            import os
+            import uuid
 
-           client = datastore.Client()
+            from google.cloud import datastore
+            from tests.system.test_system import Config  # system tests
 
-           key = client.key('_Doctest')
-           entity1 = datastore.Entity(key=key)
-           entity1['foo'] = 1337
-           entity2 = datastore.Entity(key=key)
-           entity2['foo'] = 42
-           Config.TO_DELETE.extend([entity1, entity2])
-           client.put_multi([entity1, entity2])
+            unique = os.getenv('CIRCLE_BUILD_NUM', str(uuid.uuid4())[0:8])
+            client = datastore.Client(namespace='ns{}'.format(unique))
 
-           query = client.query(kind='_Doctest')
-           cursor = None
+            key = client.key('_Doctest')
+            entity1 = datastore.Entity(key=key)
+            entity1['foo'] = 1337
+            entity2 = datastore.Entity(key=key)
+            entity2['foo'] = 42
+            Config.TO_DELETE.extend([entity1, entity2])
+            client.put_multi([entity1, entity2])
+
+            query = client.query(kind='_Doctest')
+            cursor = None
 
         .. doctest:: query-page
 
-           >>> query_iter = query.fetch(start_cursor=cursor)
-           >>> pages = query_iter.pages
-           >>>
-           >>> first_page = next(pages)
-           >>> first_page_entities = list(first_page)
-           >>> query_iter.next_page_token
-           b'...'
+            >>> query_iter = query.fetch(start_cursor=cursor)
+            >>> pages = query_iter.pages
+            >>>
+            >>> first_page = next(pages)
+            >>> first_page_entities = list(first_page)
+            >>> query_iter.next_page_token is None
+            True
 
-        :type kwargs: dict
         :param kwargs: Parameters for initializing and instance of
                        :class:`~google.cloud.datastore.query.Query`.
 
         :rtype: :class:`~google.cloud.datastore.query.Query`
         :returns: A query object.
         """
-        if 'client' in kwargs:
-            raise TypeError('Cannot pass client')
-        if 'project' in kwargs:
-            raise TypeError('Cannot pass project')
-        kwargs['project'] = self.project
-        if 'namespace' not in kwargs:
-            kwargs['namespace'] = self.namespace
+        if "client" in kwargs:
+            raise TypeError("Cannot pass client")
+        if "project" in kwargs:
+            raise TypeError("Cannot pass project")
+        kwargs["project"] = self.project
+        if "namespace" not in kwargs:
+            kwargs["namespace"] = self.namespace
         return Query(self, **kwargs)
 
+    def reserve_ids(self, complete_key, num_ids):
+        """Reserve a list of IDs from a complete key.
 
-def _get_read_options(eventual, transaction_id):
-    """Validate rules for read options, and assign to the request.
+        :type complete_key: :class:`google.cloud.datastore.key.Key`
+        :param complete_key: Partial key to use as base for reserved IDs.
 
-    Helper method for ``lookup()`` and ``run_query``.
+        :type num_ids: int
+        :param num_ids: The number of IDs to reserve.
 
-    :type eventual: bool
-    :param eventual: Flag indicating if ``EVENTUAL`` or ``STRONG``
-                     consistency should be used.
+        :rtype: class:`NoneType`
+        :returns: None
+        :raises: :class:`ValueError` if `complete_key`` is not a
+                 Complete key.
+        """
+        if complete_key.is_partial:
+            raise ValueError(("Key is not Complete.", complete_key))
 
-    :type transaction_id: bytes
-    :param transaction_id: A transaction identifier (may be null).
+        if not isinstance(num_ids, int):
+            raise ValueError(("num_ids is not a valid integer.", num_ids))
 
-    :rtype: :class:`.datastore_pb2.ReadOptions`
-    :returns: The read options corresponding to the inputs.
-    :raises: :class:`ValueError` if ``eventual`` is ``True`` and the
-             ``transaction_id`` is not ``None``.
-    """
-    if transaction_id is None:
-        if eventual:
-            return _datastore_pb2.ReadOptions(
-                read_consistency=_datastore_pb2.ReadOptions.EVENTUAL)
-        else:
-            return _datastore_pb2.ReadOptions()
-    else:
-        if eventual:
-            raise ValueError('eventual must be False when in a transaction')
-        else:
-            return _datastore_pb2.ReadOptions(
-                transaction=transaction_id)
+        complete_key_pb = complete_key.to_protobuf()
+        complete_key_pbs = [complete_key_pb] * num_ids
+
+        self._datastore_api.reserve_ids(complete_key.project, complete_key_pbs)
+
+        return None
