@@ -1,4 +1,4 @@
-# Copyright 2014 Google Inc.
+# Copyright 2014 Google LLC
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -18,18 +18,25 @@ import mock
 
 
 class TestTransaction(unittest.TestCase):
-
     @staticmethod
     def _get_target_class():
         from google.cloud.datastore.transaction import Transaction
 
         return Transaction
 
+    def _get_options_class(self, **kw):
+        from google.cloud.datastore_v1.types import TransactionOptions
+
+        return TransactionOptions
+
     def _make_one(self, client, **kw):
         return self._get_target_class()(client, **kw)
 
+    def _make_options(self, **kw):
+        return self._get_options_class()(**kw)
+
     def test_ctor_defaults(self):
-        project = 'PROJECT'
+        project = "PROJECT"
         client = _Client(project)
         xact = self._make_one(client)
         self.assertEqual(xact.project, project)
@@ -40,9 +47,9 @@ class TestTransaction(unittest.TestCase):
         self.assertEqual(len(xact._partial_key_entities), 0)
 
     def test_current(self):
-        from google.cloud.proto.datastore.v1 import datastore_pb2
+        from google.cloud.datastore_v1.proto import datastore_pb2
 
-        project = 'PROJECT'
+        project = "PROJECT"
         id_ = 678
         ds_api = _make_datastore_api(xact_id=id_)
         client = _Client(project, datastore_api=ds_api)
@@ -78,7 +85,7 @@ class TestTransaction(unittest.TestCase):
         begin_txn.assert_called_with(project)
 
     def test_begin(self):
-        project = 'PROJECT'
+        project = "PROJECT"
         id_ = 889
         ds_api = _make_datastore_api(xact_id=id_)
         client = _Client(project, datastore_api=ds_api)
@@ -88,7 +95,7 @@ class TestTransaction(unittest.TestCase):
         ds_api.begin_transaction.assert_called_once_with(project)
 
     def test_begin_tombstoned(self):
-        project = 'PROJECT'
+        project = "PROJECT"
         id_ = 1094
         ds_api = _make_datastore_api(xact_id=id_)
         client = _Client(project, datastore_api=ds_api)
@@ -104,7 +111,7 @@ class TestTransaction(unittest.TestCase):
         self.assertRaises(ValueError, xact.begin)
 
     def test_begin_w_begin_transaction_failure(self):
-        project = 'PROJECT'
+        project = "PROJECT"
         id_ = 712
         ds_api = _make_datastore_api(xact_id=id_)
         ds_api.begin_transaction = mock.Mock(side_effect=RuntimeError, spec=[])
@@ -118,7 +125,7 @@ class TestTransaction(unittest.TestCase):
         ds_api.begin_transaction.assert_called_once_with(project)
 
     def test_rollback(self):
-        project = 'PROJECT'
+        project = "PROJECT"
         id_ = 239
         ds_api = _make_datastore_api(xact_id=id_)
         client = _Client(project, datastore_api=ds_api)
@@ -130,9 +137,9 @@ class TestTransaction(unittest.TestCase):
         ds_api.begin_transaction.assert_called_once_with(project)
 
     def test_commit_no_partial_keys(self):
-        from google.cloud.proto.datastore.v1 import datastore_pb2
+        from google.cloud.datastore_v1.proto import datastore_pb2
 
-        project = 'PROJECT'
+        project = "PROJECT"
         id_ = 1002930
         ds_api = _make_datastore_api(xact_id=id_)
         client = _Client(project, datastore_api=ds_api)
@@ -142,15 +149,16 @@ class TestTransaction(unittest.TestCase):
 
         mode = datastore_pb2.CommitRequest.TRANSACTIONAL
         client._datastore_api.commit.assert_called_once_with(
-            project, mode, [], transaction=id_)
+            project, mode, [], transaction=id_
+        )
         self.assertIsNone(xact.id)
         ds_api.begin_transaction.assert_called_once_with(project)
 
     def test_commit_w_partial_keys(self):
-        from google.cloud.proto.datastore.v1 import datastore_pb2
+        from google.cloud.datastore_v1.proto import datastore_pb2
 
-        project = 'PROJECT'
-        kind = 'KIND'
+        project = "PROJECT"
+        kind = "KIND"
         id1 = 123
         key = _make_key(kind, id1, project)
         id2 = 234
@@ -164,15 +172,16 @@ class TestTransaction(unittest.TestCase):
 
         mode = datastore_pb2.CommitRequest.TRANSACTIONAL
         ds_api.commit.assert_called_once_with(
-            project, mode, xact.mutations, transaction=id2)
+            project, mode, xact.mutations, transaction=id2
+        )
         self.assertIsNone(xact.id)
-        self.assertEqual(entity.key.path, [{'kind': kind, 'id': id1}])
+        self.assertEqual(entity.key.path, [{"kind": kind, "id": id1}])
         ds_api.begin_transaction.assert_called_once_with(project)
 
     def test_context_manager_no_raise(self):
-        from google.cloud.proto.datastore.v1 import datastore_pb2
+        from google.cloud.datastore_v1.proto import datastore_pb2
 
-        project = 'PROJECT'
+        project = "PROJECT"
         id_ = 912830
         ds_api = _make_datastore_api(xact_id=id_)
         client = _Client(project, datastore_api=ds_api)
@@ -183,16 +192,16 @@ class TestTransaction(unittest.TestCase):
 
         mode = datastore_pb2.CommitRequest.TRANSACTIONAL
         client._datastore_api.commit.assert_called_once_with(
-            project, mode, [], transaction=id_)
+            project, mode, [], transaction=id_
+        )
         self.assertIsNone(xact.id)
         self.assertEqual(ds_api.begin_transaction.call_count, 1)
 
     def test_context_manager_w_raise(self):
-
         class Foo(Exception):
             pass
 
-        project = 'PROJECT'
+        project = "PROJECT"
         id_ = 614416
         ds_api = _make_datastore_api(xact_id=id_)
         client = _Client(project, datastore_api=ds_api)
@@ -205,16 +214,36 @@ class TestTransaction(unittest.TestCase):
                 raise Foo()
         except Foo:
             self.assertIsNone(xact.id)
-            client._datastore_api.rollback.assert_called_once_with(
-                project, id_)
+            client._datastore_api.rollback.assert_called_once_with(project, id_)
 
         client._datastore_api.commit.assert_not_called()
         self.assertIsNone(xact.id)
         self.assertEqual(ds_api.begin_transaction.call_count, 1)
 
+    def test_constructor_read_only(self):
+        project = "PROJECT"
+        id_ = 850302
+        ds_api = _make_datastore_api(xact=id_)
+        client = _Client(project, datastore_api=ds_api)
+        read_only = self._get_options_class().ReadOnly()
+        options = self._make_options(read_only=read_only)
+        xact = self._make_one(client, read_only=True)
+        self.assertEqual(xact._options, options)
+
+    def test_put_read_only(self):
+        project = "PROJECT"
+        id_ = 943243
+        ds_api = _make_datastore_api(xact_id=id_)
+        client = _Client(project, datastore_api=ds_api)
+        entity = _Entity()
+        xact = self._make_one(client, read_only=True)
+        xact.begin()
+        with self.assertRaises(RuntimeError):
+            xact.put(entity)
+
 
 def _make_key(kind, id_, project):
-    from google.cloud.proto.datastore.v1 import entity_pb2
+    from google.cloud.datastore_v1.proto import entity_pb2
 
     key = entity_pb2.Key()
     key.partition_id.project_id = project
@@ -225,16 +254,14 @@ def _make_key(kind, id_, project):
 
 
 class _Entity(dict):
-
     def __init__(self):
         super(_Entity, self).__init__()
         from google.cloud.datastore.key import Key
 
-        self.key = Key('KIND', project='PROJECT')
+        self.key = Key("KIND", project="PROJECT")
 
 
 class _Client(object):
-
     def __init__(self, project, datastore_api=None, namespace=None):
         self.project = project
         if datastore_api is None:
@@ -255,7 +282,6 @@ class _Client(object):
 
 
 class _NoCommitBatch(object):
-
     def __init__(self, client):
         from google.cloud.datastore.batch import Batch
 
@@ -271,22 +297,21 @@ class _NoCommitBatch(object):
 
 
 def _make_commit_response(*keys):
-    from google.cloud.proto.datastore.v1 import datastore_pb2
+    from google.cloud.datastore_v1.proto import datastore_pb2
 
-    mutation_results = [
-        datastore_pb2.MutationResult(key=key) for key in keys]
+    mutation_results = [datastore_pb2.MutationResult(key=key) for key in keys]
     return datastore_pb2.CommitResponse(mutation_results=mutation_results)
 
 
 def _make_datastore_api(*keys, **kwargs):
-    commit_method = mock.Mock(
-        return_value=_make_commit_response(*keys), spec=[])
+    commit_method = mock.Mock(return_value=_make_commit_response(*keys), spec=[])
 
-    xact_id = kwargs.pop('xact_id', 123)
-    txn_pb = mock.Mock(
-        transaction=xact_id, spec=['transaction'])
+    xact_id = kwargs.pop("xact_id", 123)
+    txn_pb = mock.Mock(transaction=xact_id, spec=["transaction"])
     begin_txn = mock.Mock(return_value=txn_pb, spec=[])
 
     return mock.Mock(
-        commit=commit_method, begin_transaction=begin_txn,
-        spec=['begin_transaction', 'commit', 'rollback'])
+        commit=commit_method,
+        begin_transaction=begin_txn,
+        spec=["begin_transaction", "commit", "rollback"],
+    )
